using System;

namespace SpectralNet
{
	/// <summary>
	/// Intermediate structure for storing uploaded data
	/// </summary>
	public class NodeDoubleVector 
	{
		public double[] Values;
		public string correspondingNodeLabel;
		public Vertex correspondingNode;
		public double sortValue;

		public NodeDoubleVector (int numComponents)
		{
			Values = new double[numComponents];
			correspondingNodeLabel = "";
			correspondingNode = null;
		}

		public NodeDoubleVector (string nodeLabel, int numComponents)
		{
			Values = new double[numComponents];
			correspondingNodeLabel = nodeLabel;
			correspondingNode = null;
		}

		public double Magnitude
		{
			get 
			{
				double mag = 0;
				for(int i = 0; i < Values.Length; i++)
				{
					mag += Math.Pow(Values[i], 2);
				}
				mag = Math.Sqrt(mag);
				return mag;
			}
		}


		public double distance(NodeDoubleVector b)
		{
			double result = 0;

			for(int i = 0; i < Values.Length; i++)
			{
				result += Math.Pow((this.Values[i] - b.Values[i]), 2);
			}

			result = Math.Sqrt(result);

			return result;
		}

		public void normalize()
		{
			double mag = this.Magnitude;
			for(int i = 0; i < Values.Length; i++)
			{
				Values[i] = Values[i] / mag;
			}
		}

		public double calculateCorrelation(NodeDoubleVector b)
		{
			if (this.Values.Length != b.Values.Length)
			{
				throw new Exception("Cannot calculate correlation between two vectors of different size.");
			}

			double result = 0;
			double n = this.Values.Length;

			double sumX = 0;
			double sumY = 0;
			double sumXY = 0;
			double sumXSquared = 0;
			double sumYSquared = 0;

			for(int i = 0; i < n; i++)
			{
				double x = this.Values[i];
				double y = b.Values[i];

				sumX += x;
				sumY += y;
				sumXY += (x * y);
				sumXSquared += (x * x);
				sumYSquared += (y * y);
			}

			double numerator = n*sumXY - (sumX * sumY);
			double denominator = Math.Sqrt((n*sumXSquared - (sumX * sumX)) * (n*sumYSquared - (sumY * sumY)));

			result = numerator / denominator;

			return result;
		}

		public double calculateDistance(NodeDoubleVector b)
		{
			if (this.Values.Length != b.Values.Length)
			{
				throw new Exception("Cannot calculate distance between two vectors of different size.");
			}

			double radicand = 0;

			for(int i = 0; i < this.Values.Length; i++)
			{
				double x = this.Values[i];
				double y = b.Values[i];

				radicand += (x - y) * (x - y);
			}

			return Math.Sqrt(radicand);
		}
	}
}
